@@ -28,6 +28,8 @@
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-ontologies.h>
 
+#include <libtracker-miner/tracker-miner-common.h>
+
 #include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-update.h"
@@ -151,6 +153,8 @@ static TrackerDataUpdateBuffer update_buffer;
 static TrackerDataUpdateBufferResource *resource_buffer;
 static TrackerDataBlankBuffer blank_buffer;
 static time_t resource_time = 0;
+static gint transaction_modseq = 0;
+static gboolean has_persistent = TRUE;
 
 static GPtrArray *insert_callbacks = NULL;
 static GPtrArray *delete_callbacks = NULL;
@@ -158,12 +162,12 @@ static GPtrArray *commit_callbacks = NULL;
 static GPtrArray *rollback_callbacks = NULL;
 static gint max_service_id = 0;
 static gint max_ontology_id = 0;
-static gint max_modseq = 0;
 
 static gint         ensure_resource_id      (const gchar      *uri,
                                              gboolean         *create);
 static void         cache_insert_value      (const gchar      *table_name,
                                              const gchar      *field_name,
+                                             gboolean          transient,
                                              GValue           *value,
                                              gint              graph,
                                              gboolean          multiple_values,
@@ -412,14 +416,6 @@ tracker_data_update_get_new_service_id (void)
 	}
 }
 
-void
-tracker_data_update_shutdown (void)
-{
-	max_service_id = 0;
-	max_ontology_id = 0;
-	max_modseq = 0;
-}
-
 static gint
 tracker_data_update_get_next_modseq (void)
 {
@@ -427,10 +423,7 @@ tracker_data_update_get_next_modseq (void)
 	TrackerDBInterface *temp_iface;
 	TrackerDBStatement *stmt;
 	GError             *error = NULL;
-
-	if (G_LIKELY (max_modseq != 0)) {
-		return ++max_modseq;
-	}
+	gint                max_modseq = 0;
 
 	temp_iface = tracker_db_manager_get_db_interface ();
 
@@ -458,6 +451,28 @@ tracker_data_update_get_next_modseq (void)
 	return ++max_modseq;
 }
 
+void
+tracker_data_update_shutdown (void)
+{
+	max_service_id = 0;
+	max_ontology_id = 0;
+	transaction_modseq = 0;
+}
+
+static gint
+get_transaction_modseq (void)
+{
+	if (G_UNLIKELY (transaction_modseq == 0)) {
+		transaction_modseq = tracker_data_update_get_next_modseq ();
+	}
+
+	/* Always use 1 for ontology transactions */
+	if (in_ontology_transaction) {
+		return 1;
+	}
+
+	return transaction_modseq;
+}
 
 static TrackerDataUpdateBufferTable *
 cache_table_new (gboolean multiple_values)
@@ -487,12 +502,13 @@ cache_table_free (TrackerDataUpdateBufferTable *table)
 }
 
 static TrackerDataUpdateBufferTable *
-cache_ensure_table (const gchar            *table_name,
-                    gboolean                multiple_values)
+cache_ensure_table (const gchar *table_name,
+                    gboolean     multiple_values,
+                    gboolean     transient)
 {
 	TrackerDataUpdateBufferTable *table;
 
-	if (!resource_buffer->modified) {
+	if (!resource_buffer->modified && !transient) {
 		/* first modification of this particular resource, update tracker:modified */
 
 		GValue gvalue = { 0 };
@@ -500,10 +516,10 @@ cache_ensure_table (const gchar            *table_name,
 		resource_buffer->modified = TRUE;
 
 		g_value_init (&gvalue, G_TYPE_INT64);
-		g_value_set_int64 (&gvalue, tracker_data_update_get_next_modseq ());
-		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue,
-			            0,
-			            FALSE, FALSE, FALSE);
+		g_value_set_int64 (&gvalue, get_transaction_modseq ());
+		cache_insert_value ("rdfs:Resource", "tracker:modified", TRUE, &gvalue,
+		                    0,
+		                    FALSE, FALSE, FALSE);
 	}
 
 	table = g_hash_table_lookup (resource_buffer->tables, table_name);
@@ -519,9 +535,9 @@ cache_ensure_table (const gchar            *table_name,
 static void
 cache_insert_row (TrackerClass *class)
 {
-	TrackerDataUpdateBufferTable    *table;
+	TrackerDataUpdateBufferTable *table;
 
-	table = cache_ensure_table (tracker_class_get_name (class), FALSE);
+	table = cache_ensure_table (tracker_class_get_name (class), FALSE, FALSE);
 	table->class = class;
 	table->insert = TRUE;
 }
@@ -529,6 +545,7 @@ cache_insert_row (TrackerClass *class)
 static void
 cache_insert_value (const gchar            *table_name,
                     const gchar            *field_name,
+                    gboolean                transient,
                     GValue                 *value,
                     gint                    graph,
                     gboolean                multiple_values,
@@ -549,7 +566,7 @@ cache_insert_value (const gchar            *table_name,
 #endif
 	property.date_time = date_time;
 
-	table = cache_ensure_table (table_name, multiple_values);
+	table = cache_ensure_table (table_name, multiple_values, transient);
 	g_array_append_val (table->properties, property);
 }
 
@@ -558,7 +575,7 @@ cache_delete_row (TrackerClass *class)
 {
 	TrackerDataUpdateBufferTable    *table;
 
-	table = cache_ensure_table (tracker_class_get_name (class), FALSE);
+	table = cache_ensure_table (tracker_class_get_name (class), FALSE, FALSE);
 	table->class = class;
 	table->delete_row = TRUE;
 }
@@ -566,6 +583,7 @@ cache_delete_row (TrackerClass *class)
 static void
 cache_delete_value (const gchar            *table_name,
                     const gchar            *field_name,
+                    gboolean                transient,
                     GValue                 *value,
                     gboolean                multiple_values,
                     gboolean                fts,
@@ -582,7 +600,7 @@ cache_delete_value (const gchar            *table_name,
 #endif
 	property.date_time = date_time;
 
-	table = cache_ensure_table (table_name, multiple_values);
+	table = cache_ensure_table (table_name, multiple_values, transient);
 	table->delete_value = TRUE;
 	g_array_append_val (table->properties, property);
 }
@@ -883,7 +901,7 @@ tracker_data_resource_buffer_flush (GError **error)
 				if (strcmp (table_name, "rdfs:Resource") == 0) {
 					g_warn_if_fail	(resource_time != 0);
 					tracker_db_statement_bind_int (stmt, 1, (gint64) resource_time);
-					tracker_db_statement_bind_int (stmt, 2, tracker_data_update_get_next_modseq ());
+					tracker_db_statement_bind_int (stmt, 2, get_transaction_modseq ());
 					param = 3;
 				} else {
 					param = 1;
@@ -1022,6 +1040,7 @@ tracker_data_update_buffer_clear (void)
 
 	g_hash_table_remove_all (update_buffer.resources);
 	g_hash_table_remove_all (update_buffer.resources_by_id);
+	g_hash_table_remove_all (update_buffer.resource_cache);
 	resource_buffer = NULL;
 
 #if HAVE_TRACKER_FTS
@@ -1154,7 +1173,7 @@ cache_create_service_decomposed (TrackerClass *cl,
 	class_id = tracker_class_get_id (cl);
 
 	g_value_set_int64 (&gvalue, class_id);
-	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue,
+	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", FALSE, &gvalue,
 	                    final_graph_id,
 	                    TRUE, FALSE, FALSE);
 
@@ -1216,6 +1235,7 @@ cache_create_service_decomposed (TrackerClass *cl,
 
 			cache_insert_value (tracker_class_get_name (cl),
 			                    tracker_property_get_name (*domain_indexes),
+			                    tracker_property_get_transient (*domain_indexes),
 			                    &gvalue_copy,
 			                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
 			                    tracker_property_get_multiple_values (*domain_indexes),
@@ -1590,7 +1610,9 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		g_value_unset (&gvalue);
 
 	} else {
-		cache_insert_value (table_name, field_name, &gvalue,
+		cache_insert_value (table_name, field_name,
+		                    tracker_property_get_transient (property),
+		                    &gvalue,
 		                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
 		                    multiple_values,
 		                    tracker_property_get_fulltext_indexed (property),
@@ -1607,7 +1629,10 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 					g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
 					g_value_copy (&gvalue, &gvalue_copy);
 
-					cache_insert_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy,
+					cache_insert_value (tracker_class_get_name (*domain_index_classes),
+					                    field_name,
+					                    tracker_property_get_transient (property),
+					                    &gvalue_copy,
 					                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
 					                    multiple_values,
 					                    tracker_property_get_fulltext_indexed (property),
@@ -1664,7 +1689,9 @@ delete_metadata_decomposed (TrackerProperty  *property,
 		/* value not found */
 		g_value_unset (&gvalue);
 	} else {
-		cache_delete_value (table_name, field_name, &gvalue, multiple_values,
+		cache_delete_value (table_name, field_name,
+		                    tracker_property_get_transient (property),
+		                    &gvalue, multiple_values,
 		                    tracker_property_get_fulltext_indexed (property),
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 
@@ -1678,7 +1705,10 @@ delete_metadata_decomposed (TrackerProperty  *property,
 					GValue gvalue_copy = { 0 };
 					g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
 					g_value_copy (&gvalue, &gvalue_copy);
-					cache_delete_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy, multiple_values,
+					cache_delete_value (tracker_class_get_name (*domain_index_classes),
+					                    field_name,
+					                    tracker_property_get_transient (property),
+					                    &gvalue_copy, multiple_values,
 					                    tracker_property_get_fulltext_indexed (property),
 					                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 				}
@@ -1829,7 +1859,9 @@ cache_delete_resource_type (TrackerClass *class,
 			g_value_copy (old_gvalue, &gvalue);
 
 			value_set_remove_value (old_values, &gvalue);
-			cache_delete_value (table_name, field_name, &gvalue, multiple_values,
+			cache_delete_value (table_name, field_name,
+			                    tracker_property_get_transient (prop),
+			                    &gvalue, multiple_values,
 			                    tracker_property_get_fulltext_indexed (prop),
 			                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
 
@@ -1843,7 +1875,10 @@ cache_delete_resource_type (TrackerClass *class,
 						GValue gvalue_copy = { 0 };
 						g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
 						g_value_copy (&gvalue, &gvalue_copy);
-						cache_delete_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy, multiple_values,
+						cache_delete_value (tracker_class_get_name (*domain_index_classes),
+						                    field_name,
+						                    tracker_property_get_transient (prop),
+						                    &gvalue_copy, multiple_values,
 						                    tracker_property_get_fulltext_indexed (prop),
 						                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
 					}
@@ -1973,7 +2008,6 @@ tracker_data_delete_statement (const gchar  *graph,
                                GError      **error)
 {
 	TrackerClass       *class;
-	TrackerProperty    *field;
 	gint                subject_id = 0;
 	gboolean            change = FALSE;
 
@@ -1994,6 +2028,8 @@ tracker_data_delete_statement (const gchar  *graph,
 	if (object && g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
 		class = tracker_ontologies_get_class_by_uri (object);
 		if (class != NULL) {
+			has_persistent = TRUE;
+
 			if (!in_journal_replay) {
 				tracker_db_journal_append_delete_statement_id (
 				       (graph != NULL ? query_resource_id (graph) : 0),
@@ -2009,11 +2045,16 @@ tracker_data_delete_statement (const gchar  *graph,
 	} else {
 		gint pred_id = 0, graph_id = 0, object_id = 0;
 		gboolean tried = FALSE;
+		TrackerProperty *field;
 
 		field = tracker_ontologies_get_property_by_uri (predicate);
 		if (field != NULL) {
+			if (!tracker_property_get_transient (field)) {
+				has_persistent = TRUE;
+			}
+
 			change = delete_metadata_decomposed (field, object, 0, error);
-			if (!in_journal_replay && change) {
+			if (!in_journal_replay && change && !tracker_property_get_transient (field)) {
 				if (tracker_property_get_data_type (field) == TRACKER_PROPERTY_TYPE_RESOURCE) {
 
 					pred_id = tracker_property_get_id (field);
@@ -2031,10 +2072,22 @@ tracker_data_delete_statement (const gchar  *graph,
 					object_id = 0;
 					tried = TRUE;
 
-					tracker_db_journal_append_delete_statement (graph_id,
-					                                            resource_buffer->id,
-					                                            pred_id,
-					                                            object);
+					if (!tracker_property_get_force_journal (field) &&
+					    g_strcmp0 (graph, TRACKER_MINER_FS_GRAPH_URN) == 0) {
+						/* do not journal this statement extracted from filesystem */
+						TrackerProperty *damaged;
+
+						damaged = tracker_ontologies_get_property_by_uri (TRACKER_TRACKER_PREFIX "damaged");
+						tracker_db_journal_append_insert_statement (graph_id,
+										            resource_buffer->id,
+										            tracker_property_get_id (damaged),
+										            "true");
+					} else {
+						tracker_db_journal_append_delete_statement (graph_id,
+						                                            resource_buffer->id,
+						                                            pred_id,
+						                                            object);
+					}
 				}
 			}
 		} else {
@@ -2178,6 +2231,10 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		prop_id = tracker_property_get_id (property);
 	}
 
+	if (!tracker_property_get_transient (property)) {
+		has_persistent = TRUE;
+	}
+
 	/* subjects and objects starting with `:' are anonymous blank nodes */
 	if (g_str_has_prefix (object, ":")) {
 		/* anonymous blank node used as object in a statement */
@@ -2235,7 +2292,7 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 			return;
 		}
 
-		if (!in_journal_replay) {
+		if (!in_journal_replay && !tracker_property_get_transient (property)) {
 			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
 			final_prop_id = (prop_id != 0) ? prop_id : tracker_data_query_resource_id (predicate);
 			object_id = query_resource_id (object);
@@ -2271,7 +2328,7 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		}
 	}
 
-	if (!in_journal_replay && change) {
+	if (!in_journal_replay && change && !tracker_property_get_transient (property)) {
 		tracker_db_journal_append_insert_statement_id (
 			(graph != NULL ? query_resource_id (graph) : 0),
 			resource_buffer->id,
@@ -2312,6 +2369,10 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 		pred_id = tracker_property_get_id (property);
 	}
 
+	if (!tracker_property_get_transient (property)) {
+		has_persistent = TRUE;
+	}
+
 	if (!tracker_data_insert_statement_common (graph, subject, predicate, object, &actual_error)) {
 		if (actual_error) {
 			g_propagate_error (error, actual_error);
@@ -2347,15 +2408,27 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 		}
 	}
 
-	if (!in_journal_replay && change) {
+	if (!in_journal_replay && change && !tracker_property_get_transient (property)) {
 		if (!tried) {
 			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
 			pred_id = (pred_id != 0) ? pred_id : tracker_data_query_resource_id (predicate);
 		}
-		tracker_db_journal_append_insert_statement (graph_id,
-		                                            resource_buffer->id,
-		                                            pred_id,
-		                                            object);
+		if (!tracker_property_get_force_journal (property) &&
+		    g_strcmp0 (graph, TRACKER_MINER_FS_GRAPH_URN) == 0) {
+			/* do not journal this statement extracted from filesystem */
+			TrackerProperty *damaged;
+
+			damaged = tracker_ontologies_get_property_by_uri (TRACKER_TRACKER_PREFIX "damaged");
+			tracker_db_journal_append_insert_statement (graph_id,
+				                                    resource_buffer->id,
+				                                    tracker_property_get_id (damaged),
+				                                    "true");
+		} else {
+			tracker_db_journal_append_insert_statement (graph_id,
+				                                    resource_buffer->id,
+				                                    pred_id,
+				                                    object);
+		}
 	}
 }
 
@@ -2374,6 +2447,8 @@ tracker_data_begin_transaction (GError **error)
 
 	resource_time = time (NULL);
 
+	has_persistent = FALSE;
+
 	if (update_buffer.resource_cache == NULL) {
 		update_buffer.resource_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 		/* used for normal transactions */
@@ -2433,17 +2508,39 @@ tracker_data_commit_transaction (GError **error)
 
 	tracker_data_update_buffer_flush (&actual_error);
 	if (actual_error) {
+		tracker_data_rollback_transaction ();
 		g_propagate_error (error, actual_error);
 		return;
 	}
 
-	in_transaction = FALSE;
-	in_ontology_transaction = FALSE;
+	tracker_db_interface_end_db_transaction (iface,
+	                                         &actual_error);
+
+	if (actual_error) {
+		tracker_data_rollback_transaction ();
+		g_propagate_error (error, actual_error);
+		return;
+	}
+
+	get_transaction_modseq ();
+	if (has_persistent && !in_ontology_transaction) {
+		transaction_modseq++;
+	}
 
 	if (!in_journal_replay) {
-		tracker_db_journal_commit_db_transaction ();
+		if (has_persistent || in_ontology_transaction) {
+			tracker_db_journal_commit_db_transaction ();
+		} else {
+			/* If we only had transient properties, then we must not write
+			 * anything to the journal. So we roll it back, but only the
+			 * journal's part. */
+			tracker_db_journal_rollback_transaction ();
+		}
 	}
+
 	resource_time = 0;
+	in_transaction = FALSE;
+	in_ontology_transaction = FALSE;
 
 	if (update_buffer.class_counts) {
 		/* successful transaction, no need to rollback class counts,
@@ -2458,8 +2555,6 @@ tracker_data_commit_transaction (GError **error)
 	}
 #endif
 
-	tracker_db_interface_end_db_transaction (iface);
-
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA cache_size = %d", TRACKER_DB_CACHE_SIZE_DEFAULT);
 
 	g_hash_table_remove_all (update_buffer.resources);
@@ -2486,6 +2581,7 @@ void
 tracker_data_rollback_transaction (void)
 {
 	TrackerDBInterface *iface;
+	GError *ignorable = NULL;
 
 	g_return_if_fail (in_transaction);
 
@@ -2495,28 +2591,37 @@ tracker_data_rollback_transaction (void)
 	iface = tracker_db_manager_get_db_interface ();
 
 	tracker_data_update_buffer_clear ();
-	tracker_db_interface_execute_query (iface, NULL, "ROLLBACK");
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA cache_size = %d", TRACKER_DB_CACHE_SIZE_DEFAULT);
-	tracker_db_journal_rollback_transaction ();
 
-	if (rollback_callbacks) {
-		guint n;
-		for (n = 0; n < rollback_callbacks->len; n++) {
-			TrackerCommitDelegate *delegate;
-			delegate = g_ptr_array_index (rollback_callbacks, n);
-			delegate->callback (TRUE, delegate->user_data);
+	tracker_db_interface_execute_query (iface, &ignorable, "ROLLBACK");
+
+	if (ignorable) {
+		g_error_free (ignorable);
+	}
+
+	tracker_db_interface_execute_query (iface, NULL, "PRAGMA cache_size = %d", TRACKER_DB_CACHE_SIZE_DEFAULT);
+
+	if (!in_journal_replay) {
+		tracker_db_journal_rollback_transaction ();
+
+		if (rollback_callbacks) {
+			guint n;
+			for (n = 0; n < rollback_callbacks->len; n++) {
+				TrackerCommitDelegate *delegate;
+				delegate = g_ptr_array_index (rollback_callbacks, n);
+				delegate->callback (TRUE, delegate->user_data);
+			}
 		}
 	}
 }
 
-static GPtrArray *
+static GVariant *
 update_sparql (const gchar  *update,
                gboolean      blank,
                GError      **error)
 {
 	GError *actual_error = NULL;
 	TrackerSparqlQuery *sparql_query;
-	GPtrArray *blank_nodes;
+	GVariant *blank_nodes;
 
 	g_return_val_if_fail (update != NULL, NULL);
 
@@ -2538,7 +2643,6 @@ update_sparql (const gchar  *update,
 
 	tracker_data_commit_transaction (&actual_error);
 	if (actual_error) {
-		tracker_data_rollback_transaction ();
 		g_propagate_error (error, actual_error);
 		return NULL;
 	}
@@ -2553,7 +2657,7 @@ tracker_data_update_sparql (const gchar  *update,
 	update_sparql (update, FALSE, error);
 }
 
-GPtrArray *
+GVariant *
 tracker_data_update_sparql_blank (const gchar  *update,
                                   GError      **error)
 {
@@ -2580,9 +2684,10 @@ tracker_data_sync (void)
 }
 
 void
-tracker_data_replay_journal (TrackerBusyCallback  busy_callback,
-                             gpointer             busy_user_data,
-                             const gchar         *busy_status)
+tracker_data_replay_journal (TrackerBusyCallback   busy_callback,
+                             gpointer              busy_user_data,
+                             const gchar          *busy_status,
+                             GError              **error)
 {
 	GError *journal_error = NULL;
 	TrackerProperty *rdf_type = NULL;
@@ -2632,8 +2737,14 @@ tracker_data_replay_journal (TrackerBusyCallback  busy_callback,
 
 			tracker_data_commit_transaction (&new_error);
 			if (new_error) {
-				g_warning ("Journal replay error: '%s'", new_error->message);
-				g_clear_error (&new_error);
+				/* Out of disk is an unrecoverable fatal error */
+				if (g_error_matches (new_error, TRACKER_DB_INTERFACE_ERROR, TRACKER_DB_NO_SPACE)) {
+					g_propagate_error (error, new_error);
+					return;
+				} else {
+					g_warning ("Journal replay error: '%s'", new_error->message);
+					g_clear_error (&new_error);
+				}
 			}
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
 			GError *new_error = NULL;
